/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Panic Button.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Dependencies:
// - utils.js
//


var g_aePanicButton = {
  _utils: null,
  _strBundle: null,
  _browserSession: null,
  _toolbarIconCls: [],

  _cssClass: {
    // The code for the _cssClass object is adapted from "JavaScript: The
    // Definitive Guide" 5/e by David Flanagan (O'Reilly, 2006, pp. 381-382)
    is: function (aElt, aCls) {
      if (typeof aElt == "string") aElt = document.getElementById(aElt);

      var classes = aElt.className;
      if (! classes) return false;
      if (classes == aCls) return true;

      return aElt.className.search("\\b" + aCls + "\\b") != -1;
    },

    add: function (aElt, aCls) {
      if (typeof aElt == "string") aElt = document.getElementById(aElt);
      if (this.is(aElt, aCls)) return;
      if (aElt.className) aCls = " " + aCls;
      aElt.className += aCls;
    },

    remove: function (aElt, aCls) {
      if (typeof aElt == "string") aElt = document.getElementById(aElt);
      aElt.className = aElt.className.replace(new RegExp("\\b" + aCls + "\\b\\s*", "g"), "");
    }
  },


  init: function ()
  {
    this._initToolbarIconClasses();
    this._utils = new aePanicButtonUtils();
    this._strBundle = document.getElementById("ae-panicbutton-strings");
    try {
      this._browserSession = Components.classes['aecreations@mozdev.org/panicbutton/browser-session;1'].getService(Components.interfaces.nsIBrowserSession);
    }
    catch (e) {
      alert(e);
    }

    this.applyUserPrefs();
  },


  _initToolbarIconClasses: function ()
  {
    this._toolbarIconCls[0] = "ae-panicbutton-default";
    this._toolbarIconCls[1] = "ae-panicbutton-exclamation-in-ball";
    this._toolbarIconCls[2] = "ae-panicbutton-window-with-exclamation";
    this._toolbarIconCls[3] = "ae-panicbutton-window-with-cross";
    this._toolbarIconCls[4] = "ae-panicbutton-window-minimize";
    this._toolbarIconCls[5] = "ae-panicbutton-quit";
  },


  applyUserPrefs: function ()
  {
    // Enable or disable F9 key
    var keyEnabled = this._utils.getPref("panicbutton.enable_function_key");
    var keysetElt = document.getElementById("mainKeyset");
    var panicButtonKeyElt;

    for (var i = 0; i < keysetElt.childNodes.length; i++) {
      if (keysetElt.childNodes[i].id == "ae_key_panicbutton") {
	panicButtonKeyElt = keysetElt.childNodes[i];
      }
    }

    if (keyEnabled) {
      if (! panicButtonKeyElt) {
	// The following code works, but the user must still either restart the
	// app or open a new window for the change to take effect.
	panicButtonKeyElt = document.createElement("key");
	panicButtonKeyElt.id = "ae_key_panicbutton";
	panicButtonKeyElt.setAttribute("keycode", "VK_F9");
	panicButtonKeyElt.setAttribute("command", "ae_cmd_panicbutton");
	keysetElt.appendChild(panicButtonKeyElt);
      }
    }
    else {
      if (panicButtonKeyElt) {
	keysetElt.removeChild(panicButtonKeyElt);
      }
    }

    // Set Panic Button toolbar button label
    var label = this._utils.getPref("panicbutton.toolbarbutton_label");
    var toolbarBtnElt = document.getElementById("ae-panicbutton-toolbarbutton");
    if (label == "") {
      toolbarBtnElt.label = this._strBundle.getString('panicbutton.defaultLabel');
    }
    else {
      toolbarBtnElt.label = label;
    }

    // Set Panic Button toolbar button image
    var iconIdx = this._utils.getPref("panicbutton.toolbarbutton_icon");
    var oldCls = this._getOldToolbarButtonClass(toolbarBtnElt);
    this._cssClass.remove(toolbarBtnElt, oldCls);
    this._cssClass.add(toolbarBtnElt, this._toolbarIconCls[iconIdx]);
  },


  _getOldToolbarButtonClass: function (aToolbarButtonElt)
  {
    var fullClsName = aToolbarButtonElt.className;
    var rv = fullClsName.replace(/toolbarbutton\-1 /, "");
    return rv;
  },


  doPanicAction: function ()
  {
    var action = this._utils.getPref("panicbutton.action", 0);

    if (action == this._utils.PANICBUTTON_ACTION_HIDE) {
      this._closeAll(true);
    }
    else if (action == this._utils.PANICBUTTON_ACTION_MINIMIZE) {
      this._minimizeAll();
    }
    else if (action == this._utils.PANICBUTTON_ACTION_QUIT) {
      this._closeAll(false);
    }
  },


  _closeAll: function (aSaveSession)
  {
    if (aSaveSession) {
      var ss = Components.classes["@mozilla.org/browser/sessionstore;1"]
                         .getService(Components.interfaces.nsISessionStore);
      var state = ss.getBrowserState();
      this._browserSession.data = state;
    }

    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
    // Close any open dialog boxes
    // **NOTE: The enumerator returned by nsIWindowMediator.getEnumerator()
    // will exclude system native file-picker dialogs.
    var dlgEnum = wm.getEnumerator("");

    while (dlgEnum.hasMoreElements()) {
      var dlg = dlgEnum.getNext();
      var docElt = dlg.document.documentElement;
      if (docElt.tagName == "dialog" || docElt.tagName == "prefwindow"
	  || docElt.tagName == "wizard") {
	docElt.cancelDialog();
      }
    }

    var wndEnum = wm.getEnumerator("");

    // Need to do this to make it work on Firefox 3
    window.setTimeout(g_aePanicButton._closeAllWindows, 1, aSaveSession, wndEnum);
  },

  
  _closeAllWindows: function (aSaveSession, aWndEnum)
  {
    // Close browser and ancillary app windows
    while (aWndEnum.hasMoreElements()) {
      var wnd = aWndEnum.getNext();
      wnd.close();
    }

    if (aSaveSession) {
      window.open("chrome://panicbutton/content/panicbuttonToolbar.xul", "ae_pbtb", "chrome,dialog=0,popup");
    }
  },

  
  _minimizeAll: function () 
  {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
    var wndEnum = wm.getEnumerator("");  // Get all host app windows

    while (wndEnum.hasMoreElements()) {
      var wnd = wndEnum.getNext();
      wnd = wnd.QueryInterface(Components.interfaces.nsIDOMChromeWindow);
      wnd.minimize();
    }
  }
};


//
// Initialization with host app
//

window.addEventListener("load", function () {g_aePanicButton.init()}, false);
